@@ -4,6 +4,7 @@ from dagster.config.config_schema import ConfigSchema
 from dagster.config.config_type import Array, Noneable, ScalarUnion
 from dagster.core.definitions import (
     AssetKey,
+    AssetMaterialization,
     CompositeSolidDefinition,
     ConfigMapping,
     DependencyDefinition,
@@ -13,6 +14,7 @@ from dagster.core.definitions import (
     Failure,
     InputDefinition,
     InputMapping,
+    IntermediateStorageDefinition,
     JsonMetadataEntryData,
     LoggerDefinition,
     MarkdownMetadataEntryData,
@@ -43,8 +45,10 @@ from dagster.core.definitions import (
     daily_schedule,
     default_executors,
     executor,
+    failure_hook,
     hourly_schedule,
     in_process_executor,
+    intermediate_storage,
     lambda_solid,
     logger,
     monthly_schedule,
@@ -55,9 +59,11 @@ from dagster.core.definitions import (
     resource,
     schedule,
     solid,
+    success_hook,
     system_storage,
     weekly_schedule,
 )
+from dagster.core.definitions.config_mappable import configured
 from dagster.core.errors import (
     DagsterConfigMappingFunctionError,
     DagsterError,
@@ -88,7 +94,11 @@ from dagster.core.execution.api import (
 from dagster.core.execution.context.compute import SolidExecutionContext
 from dagster.core.execution.context.init import InitResourceContext
 from dagster.core.execution.context.logger import InitLoggerContext
-from dagster.core.execution.context.system import SystemComputeExecutionContext, TypeCheckContext
+from dagster.core.execution.context.system import (
+    HookContext,
+    SystemComputeExecutionContext,
+    TypeCheckContext,
+)
 from dagster.core.execution.results import (
     CompositeSolidExecutionResult,
     PipelineExecutionResult,
@@ -97,16 +107,26 @@ from dagster.core.execution.results import (
 from dagster.core.executor.base import Executor
 from dagster.core.executor.init import InitExecutorContext
 from dagster.core.instance import DagsterInstance
+from dagster.core.launcher import DefaultRunLauncher
 from dagster.core.log_manager import DagsterLogManager
-from dagster.core.storage.file_manager import FileHandle, LocalFileHandle
+from dagster.core.storage.file_manager import FileHandle, LocalFileHandle, local_file_manager
 from dagster.core.storage.init import InitSystemStorageContext
 from dagster.core.storage.pipeline_run import PipelineRun
 from dagster.core.storage.system_storage import (
+    build_intermediate_storage_from_object_store,
+    default_intermediate_storage_defs,
     default_system_storage_defs,
+    fs_intermediate_storage,
     fs_system_storage,
+    mem_intermediate_storage,
     mem_system_storage,
 )
-from dagster.core.types.config_schema import input_hydration_config, output_materialization_config
+from dagster.core.types.config_schema import (
+    dagster_type_loader,
+    dagster_type_materializer,
+    input_hydration_config,
+    output_materialization_config,
+)
 from dagster.core.types.dagster_type import DagsterType, List, Optional, PythonObjectDagsterType
 from dagster.core.types.decorator import (
     make_python_type_usable_as_dagster_type,
@@ -132,6 +152,7 @@ from dagster.config.source import StringSource, IntSource  # isort:skip
 __all__ = [
     # Definition
     'AssetKey',
+    'AssetMaterialization',
     'CompositeSolidDefinition',
     'ConfigMapping',
     'DependencyDefinition',
@@ -142,6 +163,7 @@ __all__ = [
     'Field',
     'InputDefinition',
     'InputMapping',
+    'IntermediateStorageDefinition',
     'JsonMetadataEntryData',
     'LoggerDefinition',
     'MarkdownMetadataEntryData',
@@ -164,6 +186,7 @@ __all__ = [
     # Decorators
     'composite_solid',
     'executor',
+    'intermediate_storage',
     'lambda_solid',
     'logger',
     'pipeline',
@@ -172,10 +195,13 @@ __all__ = [
     'schedule',
     'solid',
     'system_storage',
+    'success_hook',
+    'failure_hook',
     # Execution
     'CompositeSolidExecutionResult',
     'DagsterEvent',
     'DagsterEventType',
+    'DefaultRunLauncher',
     'Executor',
     'InitExecutorContext',
     'InitLoggerContext',
@@ -187,15 +213,19 @@ __all__ = [
     'SystemComputeExecutionContext',
     'SolidExecutionContext',
     'SystemStorageData',
+    'HookContext',
     'TypeCheckContext',
     'PipelineRun',
     'default_executors',
+    'default_intermediate_storage_defs',
     'default_system_storage_defs',
     'execute_pipeline_iterator',
     'execute_pipeline',
     'execute_solid_within_pipeline',
+    'fs_intermediate_storage',
     'fs_system_storage',
     'in_process_executor',
+    'mem_intermediate_storage',
     'mem_system_storage',
     'multiprocess_executor',
     'reconstructable',
@@ -223,10 +253,12 @@ __all__ = [
     # Logging
     'DagsterLogManager',
     # Utilities
+    'build_intermediate_storage_from_object_store',
     'check_dagster_type',
     'execute_solid',
     'execute_solids_within_pipeline',
     'file_relative_path',
+    'configured',
     # types
     'Any',
     'Bool',
@@ -243,6 +275,8 @@ __all__ = [
     'String',
     'Tuple',
     'TypeCheck',
+    'dagster_type_loader',
+    'dagster_type_materializer',
     'input_hydration_config',
     'output_materialization_config',
     # type creation
@@ -263,6 +297,7 @@ __all__ = [
     # file things
     'FileHandle',
     'LocalFileHandle',
+    'local_file_manager',
     # instance
     'DagsterInstance',
     # partitions and schedules
