@@ -11,33 +11,39 @@ import yaml
 
 from dagster import PipelineDefinition, check, execute_pipeline
 from dagster.cli.workspace.cli_target import (
+    get_external_pipeline_from_external_repo,
     get_external_pipeline_from_kwargs,
     get_external_repository_from_kwargs,
+    get_external_repository_from_repo_location,
+    get_repository_location_from_kwargs,
     pipeline_target_argument,
     repository_target_argument,
 )
 from dagster.core.definitions.executable import ExecutablePipeline
-from dagster.core.definitions.partition import PartitionScheduleDefinition
-from dagster.core.host_representation import InProcessRepositoryLocation
+from dagster.core.errors import (
+    DagsterBackfillFailedError,
+    DagsterInvariantViolationError,
+    DagsterLaunchFailedError,
+)
+from dagster.core.host_representation import (
+    ExternalPipeline,
+    ExternalRepository,
+    RepositoryHandle,
+    RepositoryLocation,
+)
+from dagster.core.host_representation.external_data import ExternalPartitionExecutionErrorData
+from dagster.core.host_representation.selector import PipelineSelector
 from dagster.core.instance import DagsterInstance
 from dagster.core.snap import PipelineSnapshot, SolidInvocationSnap
+from dagster.core.snap.execution_plan_snapshot import ExecutionPlanSnapshotErrorData
 from dagster.core.storage.pipeline_run import PipelineRun
-from dagster.core.telemetry import log_repo_stats, telemetry_wrapper
+from dagster.core.telemetry import log_external_repo_stats, telemetry_wrapper
 from dagster.core.utils import make_new_backfill_id
 from dagster.seven import IS_WINDOWS, JSONDecodeError, json
-from dagster.utils import (
-    DEFAULT_REPOSITORY_YAML_FILENAME,
-    DEFAULT_WORKSPACE_YAML_FILENAME,
-    load_yaml_from_glob_list,
-    merge_dicts,
-)
+from dagster.utils import DEFAULT_WORKSPACE_YAML_FILENAME, load_yaml_from_glob_list, merge_dicts
 from dagster.utils.backcompat import canonicalize_backcompat_args
 from dagster.utils.error import serializable_error_info_from_exc_info
-from dagster.utils.hosted_user_process import (
-    recon_pipeline_from_origin,
-    recon_repo_from_external_repo,
-    repository_def_from_repository_handle,
-)
+from dagster.utils.hosted_user_process import recon_pipeline_from_origin
 from dagster.utils.indenting_printer import IndentingPrinter
 
 from .config_scaffolder import scaffold_pipeline_config
@@ -64,53 +70,23 @@ def apply_click_params(command, *click_params):
     return command
 
 
-def legacy_repository_target_argument(f):
-    return apply_click_params(
-        f,
-        click.option(
-            '--workspace', '-w', type=click.Path(exists=True), help=('Path to workspace file')
-        ),
-        click.option(
-            '--repository-yaml',
-            '-y',
-            type=click.Path(exists=True),
-            help=(
-                'Path to config file. Defaults to ./{default_filename} if --python-file '
-                'and --module-name are not specified'
-            ).format(default_filename=DEFAULT_REPOSITORY_YAML_FILENAME),
-        ),
-        click.option(
-            '--python-file',
-            '-f',
-            type=click.Path(exists=True),
-            help='Specify python file where repository or pipeline function lives.',
-        ),
-        click.option(
-            '--module-name', '-m', help='Specify module where repository or pipeline function lives'
-        ),
-        click.option('--fn-name', '-n', help='Function that returns either repository or pipeline'),
-    )
-
-
 @click.command(
     name='list',
     help="List the pipelines in a repository. {warning}".format(warning=WORKSPACE_TARGET_WARNING),
 )
 @repository_target_argument
 def pipeline_list_command(**kwargs):
-    return execute_list_command(kwargs, click.echo)
+    return execute_list_command(kwargs, click.echo, DagsterInstance.get())
 
 
-def execute_list_command(cli_args, print_fn):
-    external_repository = get_external_repository_from_kwargs(cli_args)
-    # We should move this to use external repository
-    # https://github.com/dagster-io/dagster/issues/2556
-    repository = repository_def_from_repository_handle(external_repository.handle)
-    title = 'Repository {name}'.format(name=repository.name)
+def execute_list_command(cli_args, print_fn, instance):
+    check.inst_param(instance, 'instance', DagsterInstance)
+    external_repository = get_external_repository_from_kwargs(cli_args, instance)
+    title = 'Repository {name}'.format(name=external_repository.name)
     print_fn(title)
     print_fn('*' * len(title))
     first = True
-    for pipeline in repository.get_all_pipelines():
+    for pipeline in external_repository.get_all_external_pipelines():
         pipeline_title = 'Pipeline: {name}'.format(name=pipeline.name)
 
         if not first:
@@ -122,8 +98,8 @@ def execute_list_command(cli_args, print_fn):
             print_fn('Description:')
             print_fn(format_description(pipeline.description, indent=' ' * 4))
         print_fn('Solids: (Execution Order)')
-        for solid in pipeline.solids_in_topological_order:
-            print_fn('    ' + solid.name)
+        for solid_name in pipeline.pipeline_snapshot.solid_names_in_topological_order:
+            print_fn('    ' + solid_name)
 
 
 def format_description(desc, indent):
@@ -152,8 +128,8 @@ def get_partitioned_pipeline_instructions(command_name):
     return (
         'This commands targets a partitioned pipeline. The pipeline and partition set must be '
         'defined in a repository, which can be specified in a number of ways:'
-        '\n\n1. dagster pipeline {command_name} <<pipeline_name>> (works if .{default_filename} exists)'
-        '\n\n2. dagster pipeline {command_name} <<pipeline_name>> -w path/to/{default_filename}'
+        '\n\n1. dagster pipeline {command_name} -p <<pipeline_name>> (works if .{default_filename} exists)'
+        '\n\n2. dagster pipeline {command_name} -p <<pipeline_name>> -w path/to/{default_filename}'
         '\n\n3. dagster pipeline {command_name} -f /path/to/file.py -a define_some_repo -p <<pipeline_name>>'
         '\n\n4. dagster pipeline {command_name} -m a_module.submodule -a define_some_repo -p <<pipeline_name>>'
     ).format(command_name=command_name, default_filename=DEFAULT_WORKSPACE_YAML_FILENAME)
@@ -172,7 +148,7 @@ def pipeline_print_command(verbose, **cli_args):
 
 
 def execute_print_command(verbose, cli_args, print_fn):
-    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
+    external_pipeline = get_external_pipeline_from_kwargs(cli_args, DagsterInstance.get())
     pipeline_snapshot = external_pipeline.pipeline_snapshot
 
     if verbose:
@@ -265,7 +241,7 @@ def print_solid(printer, pipeline_snapshot, solid_invocation_snap):
     'preset_defs.',
 )
 @click.option(
-    '-d', '--mode', type=click.STRING, help='The name of the mode in which to execute the pipeline.'
+    '--mode', type=click.STRING, help='The name of the mode in which to execute the pipeline.'
 )
 @click.option('--tags', type=click.STRING, help='JSON string of tags to use for this pipeline run')
 @click.option(
@@ -294,8 +270,13 @@ def print_solid(printer, pipeline_snapshot, solid_invocation_snap):
         '-e, --env is deprecated and will be removed in 0.9.0.'
     ),
 )
-@telemetry_wrapper
 def pipeline_execute_command(config, preset, mode, **kwargs):
+    return _logged_pipeline_execute_command(config, preset, mode, DagsterInstance.get(), kwargs)
+
+
+@telemetry_wrapper
+def _logged_pipeline_execute_command(config, preset, mode, instance, kwargs):
+    check.inst_param(instance, 'instance', DagsterInstance)
     env = (
         canonicalize_backcompat_args(
             (config if config else None),
@@ -313,27 +294,38 @@ def pipeline_execute_command(config, preset, mode, **kwargs):
     if preset:
         if env:
             raise click.UsageError('Can not use --preset with --env.')
-        return execute_execute_command_with_preset(preset, kwargs, mode)
+        return execute_execute_command_with_preset(preset, kwargs, instance, mode)
 
     env = list(env)
     tags = get_tags_from_args(kwargs)
 
-    result = execute_execute_command(env, kwargs, mode, tags)
+    result = execute_execute_command(env, kwargs, instance, mode, tags)
     if not result.success:
         raise click.ClickException('Pipeline run {} resulted in failure.'.format(result.run_id))
 
 
-def execute_execute_command(env, cli_args, mode=None, tags=None):
-    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
+def get_run_config_from_env_file_list(env_file_list):
+    check.opt_list_param(env_file_list, 'env_file_list', of_type=str)
+    return load_yaml_from_glob_list(env_file_list) if env_file_list else {}
+
+
+def execute_execute_command(env_file_list, cli_args, instance=None, mode=None, tags=None):
+    check.opt_list_param(env_file_list, 'env_file_list', of_type=str)
+    check.opt_inst_param(instance, 'instance', DagsterInstance)
+
+    instance = instance if instance else DagsterInstance.get()
+    external_pipeline = get_external_pipeline_from_kwargs(cli_args, instance)
     # We should move this to use external pipeline
     # https://github.com/dagster-io/dagster/issues/2556
     pipeline = recon_pipeline_from_origin(external_pipeline.handle.get_origin())
     solid_selection = get_solid_selection_from_args(cli_args)
-    return do_execute_command(pipeline, env, mode, tags, solid_selection)
+    return do_execute_command(pipeline, instance, env_file_list, mode, tags, solid_selection)
 
 
-def execute_execute_command_with_preset(preset_name, cli_args, _mode):
-    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
+def execute_execute_command_with_preset(preset, cli_args, instance, _mode):
+    instance = instance if instance else DagsterInstance.get()
+
+    external_pipeline = get_external_pipeline_from_kwargs(cli_args, instance)
     # We should move this to use external pipeline
     # https://github.com/dagster-io/dagster/issues/2556
     pipeline = recon_pipeline_from_origin(external_pipeline.handle.get_origin())
@@ -343,26 +335,190 @@ def execute_execute_command_with_preset(preset_name, cli_args, _mode):
 
     return execute_pipeline(
         pipeline,
-        preset=preset_name,
-        instance=DagsterInstance.get(),
+        preset=preset,
+        instance=instance,
         raise_on_error=False,
         tags=tags,
         solid_selection=solid_selection,
     )
 
 
-def do_execute_command(pipeline, env_file_list, mode=None, tags=None, solid_selection=None):
-    check.inst_param(pipeline, 'pipeline', ExecutablePipeline)
-    env_file_list = check.opt_list_param(env_file_list, 'env_file_list', of_type=str)
+def _check_execute_external_pipeline_args(
+    external_pipeline, run_config, mode, preset, tags, solid_selection
+):
+    check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
+    run_config = check.opt_dict_param(run_config, 'run_config')
+    check.opt_str_param(mode, 'mode')
+    check.opt_str_param(preset, 'preset')
+    check.invariant(
+        not (mode is not None and preset is not None),
+        'You may set only one of `mode` (got {mode}) or `preset` (got {preset}).'.format(
+            mode=mode, preset=preset
+        ),
+    )
 
-    run_config = load_yaml_from_glob_list(env_file_list) if env_file_list else {}
+    tags = check.opt_dict_param(tags, 'tags', key_type=str)
+    check.opt_list_param(solid_selection, 'solid_selection', of_type=str)
+
+    if preset is not None:
+        pipeline_preset = external_pipeline.get_preset(preset)
+
+        if pipeline_preset.run_config is not None:
+            check.invariant(
+                (not run_config) or (pipeline_preset.run_config == run_config),
+                'The environment set in preset \'{preset}\' does not agree with the environment '
+                'passed in the `run_config` argument.'.format(preset=preset),
+            )
+
+            run_config = pipeline_preset.run_config
+
+        # load solid_selection from preset
+        if pipeline_preset.solid_selection is not None:
+            check.invariant(
+                solid_selection is None or solid_selection == pipeline_preset.solid_selection,
+                'The solid_selection set in preset \'{preset}\', {preset_subset}, does not agree with '
+                'the `solid_selection` argument: {solid_selection}'.format(
+                    preset=preset,
+                    preset_subset=pipeline_preset.solid_selection,
+                    solid_selection=solid_selection,
+                ),
+            )
+            solid_selection = pipeline_preset.solid_selection
+
+        check.invariant(
+            mode is None or mode == pipeline_preset.mode,
+            'Mode {mode} does not agree with the mode set in preset \'{preset}\': '
+            '(\'{preset_mode}\')'.format(
+                preset=preset, preset_mode=pipeline_preset.mode, mode=mode
+            ),
+        )
+
+        mode = pipeline_preset.mode
+
+        tags = merge_dicts(pipeline_preset.tags, tags)
+
+    if mode is not None:
+        if not external_pipeline.has_mode(mode):
+            raise DagsterInvariantViolationError(
+                (
+                    'You have attempted to execute pipeline {name} with mode {mode}. '
+                    'Available modes: {modes}'
+                ).format(
+                    name=external_pipeline.name, mode=mode, modes=external_pipeline.available_modes,
+                )
+            )
+    else:
+        if len(external_pipeline.available_modes) > 1:
+            raise DagsterInvariantViolationError(
+                (
+                    'Pipeline {name} has multiple modes (Available modes: {modes}) and you have '
+                    'attempted to execute it without specifying a mode. Set '
+                    'mode property on the PipelineRun object.'
+                ).format(name=external_pipeline.name, modes=external_pipeline.available_modes)
+            )
+        mode = external_pipeline.get_default_mode_name()
+
+    tags = merge_dicts(external_pipeline.tags, tags)
+
+    return (
+        run_config,
+        mode,
+        tags,
+        solid_selection,
+    )
+
+
+def _create_external_pipeline_run(
+    instance,
+    repo_location,
+    external_repo,
+    external_pipeline,
+    run_config,
+    mode,
+    preset,
+    tags,
+    solid_selection,
+):
+    check.inst_param(instance, 'instance', DagsterInstance)
+    check.inst_param(repo_location, 'repo_location', RepositoryLocation)
+    check.inst_param(external_repo, 'external_repo', ExternalRepository)
+    check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
+    check.opt_dict_param(run_config, 'run_config')
+
+    check.opt_str_param(mode, 'mode')
+    check.opt_str_param(preset, 'preset')
+    check.opt_dict_param(tags, 'tags', key_type=str)
+    check.opt_list_param(solid_selection, 'solid_selection', of_type=str)
+
+    run_config, mode, tags, solid_selection = _check_execute_external_pipeline_args(
+        external_pipeline, run_config, mode, preset, tags, solid_selection,
+    )
+
+    pipeline_name = external_pipeline.name
+    pipeline_selector = PipelineSelector(
+        location_name=repo_location.name,
+        repository_name=external_repo.name,
+        pipeline_name=pipeline_name,
+        solid_selection=solid_selection,
+    )
+
+    subset_pipeline_result = repo_location.get_subset_external_pipeline_result(pipeline_selector)
+    if subset_pipeline_result.success == False:
+        raise DagsterLaunchFailedError(
+            'Failed to load external pipeline subset: {error_message}'.format(
+                error_message=subset_pipeline_result.error.message
+            ),
+            serializable_error_info=subset_pipeline_result.error,
+        )
+
+    external_pipeline_subset = ExternalPipeline(
+        subset_pipeline_result.external_pipeline_data, external_repo.handle,
+    )
+
+    pipeline_mode = mode or external_pipeline_subset.get_default_mode_name()
+
+    external_execution_plan = repo_location.get_external_execution_plan(
+        external_pipeline_subset, run_config, pipeline_mode, step_keys_to_execute=None,
+    )
+    if isinstance(external_execution_plan, ExecutionPlanSnapshotErrorData):
+        raise DagsterLaunchFailedError(
+            'Failed to load external execution plan',
+            serializable_error_info=external_execution_plan.error,
+        )
+    else:
+        execution_plan_snapshot = external_execution_plan.execution_plan_snapshot
+
+    return instance.create_run(
+        pipeline_name=pipeline_name,
+        run_id=None,
+        run_config=run_config,
+        mode=pipeline_mode,
+        solids_to_execute=external_pipeline_subset.solids_to_execute,
+        step_keys_to_execute=None,
+        solid_selection=solid_selection,
+        status=None,
+        root_run_id=None,
+        parent_run_id=None,
+        tags=tags,
+        pipeline_snapshot=external_pipeline_subset.pipeline_snapshot,
+        execution_plan_snapshot=execution_plan_snapshot,
+        parent_pipeline_snapshot=external_pipeline_subset.parent_pipeline_snapshot,
+    )
+
+
+def do_execute_command(
+    pipeline, instance, env_file_list, mode=None, tags=None, solid_selection=None
+):
+    check.inst_param(pipeline, 'pipeline', ExecutablePipeline)
+    check.inst_param(instance, 'instance', DagsterInstance)
+    check.opt_list_param(env_file_list, 'env_file_list', of_type=str)
 
     return execute_pipeline(
         pipeline,
-        run_config=run_config,
+        run_config=get_run_config_from_env_file_list(env_file_list),
         mode=mode,
         tags=tags,
-        instance=DagsterInstance.get(),
+        instance=instance,
         raise_on_error=False,
         solid_selection=solid_selection,
     )
@@ -395,13 +551,13 @@ def do_execute_command(pipeline, env_file_list, mode=None, tags=None, solid_sele
     ),
 )
 @click.option(
-    '--preset-name',
+    '--preset',
     type=click.STRING,
     help='Specify a preset to use for this pipeline. Presets are defined on pipelines under '
     'preset_defs.',
 )
 @click.option(
-    '-d', '--mode', type=click.STRING, help='The name of the mode in which to execute the pipeline.'
+    '--mode', type=click.STRING, help='The name of the mode in which to execute the pipeline.'
 )
 @click.option('--tags', type=click.STRING, help='JSON string of tags to use for this pipeline run')
 @click.option(
@@ -430,8 +586,13 @@ def do_execute_command(pipeline, env_file_list, mode=None, tags=None, solid_sele
         '-e, --env is deprecated and will be removed in 0.9.0.'
     ),
 )
+def pipeline_launch_command(config, preset, mode, **kwargs):
+    return _logged_pipeline_launch_command(config, preset, mode, DagsterInstance.get(), kwargs)
+
+
 @telemetry_wrapper
-def pipeline_launch_command(config, preset_name, mode, **kwargs):
+def _logged_pipeline_launch_command(config, preset, mode, instance, kwargs):
+    check.inst_param(instance, 'instance', DagsterInstance)
     env = (
         canonicalize_backcompat_args(
             (config if config else None),
@@ -446,22 +607,26 @@ def pipeline_launch_command(config, preset_name, mode, **kwargs):
 
     env = list(check.opt_tuple_param(env, 'env', default=(), of_type=str))
 
-    external_pipeline = get_external_pipeline_from_kwargs(kwargs)
-    # We should move this to use external pipeline
-    # https://github.com/dagster-io/dagster/issues/2556
-    pipeline = recon_pipeline_from_origin(external_pipeline.get_origin())
+    repo_location = get_repository_location_from_kwargs(kwargs, instance)
+    external_repo = get_external_repository_from_repo_location(
+        repo_location, kwargs.get('repository')
+    )
+    external_pipeline = get_external_pipeline_from_external_repo(
+        external_repo, kwargs.get('pipeline'),
+    )
 
-    instance = DagsterInstance.get()
-    log_repo_stats(instance=instance, pipeline=pipeline, source='pipeline_launch_command')
+    log_external_repo_stats(
+        instance=instance,
+        external_pipeline=external_pipeline,
+        external_repo=external_repo,
+        source='pipeline_launch_command',
+    )
 
-    if preset_name:
+    if preset:
         if env:
             raise click.UsageError('Can not use --preset with --config.')
 
-        if mode:
-            raise click.UsageError('Can not use --preset with --mode.')
-
-        preset = pipeline.get_definition().get_preset(preset_name)
+        preset = external_pipeline.get_preset(preset)
     else:
         preset = None
 
@@ -469,39 +634,16 @@ def pipeline_launch_command(config, preset_name, mode, **kwargs):
 
     solid_selection = get_solid_selection_from_args(kwargs)
 
-    if preset and preset.solid_selection is not None:
-        check.invariant(
-            solid_selection is None or solid_selection == preset.solid_selection,
-            'The solid_selection set in preset \'{preset}\', {preset_subset}, does not agree with '
-            'the `solid_selection` argument: {solid_selection}'.format(
-                preset=preset,
-                preset_subset=preset.solid_selection,
-                solid_selection=solid_selection,
-            ),
-        )
-        solid_selection = preset.solid_selection
-
-    # generate pipeline subset from the given solid_selection
-    if solid_selection:
-        pipeline = pipeline.subset_for_execution(solid_selection)
-
-    # FIXME need to check the env against run_config
-    pipeline_run = instance.create_run_for_pipeline(
-        pipeline_def=pipeline.get_definition(),
-        solid_selection=solid_selection,
-        solids_to_execute=pipeline.solids_to_execute,
-        run_config=preset.run_config if preset else load_yaml_from_glob_list(env),
-        mode=(preset.mode if preset else mode) or 'default',
+    pipeline_run = _create_external_pipeline_run(
+        instance=instance,
+        repo_location=repo_location,
+        external_repo=external_repo,
+        external_pipeline=external_pipeline,
+        run_config=get_run_config_from_env_file_list(env),
+        mode=mode,
+        preset=preset,
         tags=run_tags,
-    )
-
-    recon_repo = pipeline.get_reconstructable_repository()
-
-    repo_location = InProcessRepositoryLocation(recon_repo)
-    external_pipeline = (
-        repo_location.get_repository(recon_repo.get_definition().name).get_full_external_pipeline(
-            pipeline.get_definition().name
-        ),
+        solid_selection=solid_selection,
     )
 
     return instance.launch_run(pipeline_run.run_id, external_pipeline)
@@ -520,7 +662,7 @@ def pipeline_scaffold_command(**kwargs):
 
 
 def execute_scaffold_command(cli_args, print_fn):
-    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
+    external_pipeline = get_external_pipeline_from_kwargs(cli_args, DagsterInstance.get())
     # We should move this to use external pipeline
     # https://github.com/dagster-io/dagster/issues/2556
     pipeline = recon_pipeline_from_origin(external_pipeline.get_origin())
@@ -538,7 +680,7 @@ def do_scaffold_command(pipeline_def, printer, skip_non_required):
     printer(yaml_string)
 
 
-def gen_partitions_from_args(partition_set, kwargs):
+def gen_partition_names_from_args(partition_names, kwargs):
     partition_selector_args = [
         bool(kwargs.get('all')),
         bool(kwargs.get('partitions')),
@@ -549,26 +691,24 @@ def gen_partitions_from_args(partition_set, kwargs):
             'error, cannot use more than one of: `--all`, `--partitions`, `--from/--to`'
         )
 
-    partitions = partition_set.get_partitions()
-
     if kwargs.get('all'):
-        return partitions
+        return partition_names
 
     if kwargs.get('partitions'):
         selected_args = [s.strip() for s in kwargs.get('partitions').split(',') if s.strip()]
         selected_partitions = [
-            partition for partition in partitions if partition.name in selected_args
+            partition for partition in partition_names if partition in selected_args
         ]
         if len(selected_partitions) < len(selected_args):
-            selected_names = [partition.name for partition in selected_partitions]
+            selected_names = [partition for partition in selected_partitions]
             unknown = [selected for selected in selected_args if selected not in selected_names]
             raise click.UsageError('Unknown partitions: {}'.format(unknown.join(', ')))
         return selected_partitions
 
-    start = validate_partition_slice(partitions, 'from', kwargs.get('from'))
-    end = validate_partition_slice(partitions, 'to', kwargs.get('to'))
+    start = validate_partition_slice(partition_names, 'from', kwargs.get('from'))
+    end = validate_partition_slice(partition_names, 'to', kwargs.get('to'))
 
-    return partitions[start:end]
+    return partition_names[start:end]
 
 
 def get_tags_from_args(kwargs):
@@ -599,14 +739,14 @@ def print_partition_format(partitions, indent_level):
         screen_width = min(250, int(tty_width))
     else:
         screen_width = 250
-    max_str_len = max(len(x.name) for x in partitions)
+    max_str_len = max(len(x) for x in partitions)
     spacing = 10
     num_columns = min(10, int((screen_width - indent_level) / (max_str_len + spacing)))
     column_width = int((screen_width - indent_level) / num_columns)
     prefix = ' ' * max(0, indent_level - spacing)
     lines = []
     for chunk in list(split_chunk(partitions, num_columns)):
-        lines.append(prefix + ''.join(partition.name.rjust(column_width) for partition in chunk))
+        lines.append(prefix + ''.join(partition.rjust(column_width) for partition in chunk))
 
     return '\n' + '\n'.join(lines)
 
@@ -616,11 +756,10 @@ def split_chunk(l, n):
         yield l[i : i + n]
 
 
-def validate_partition_slice(partitions, name, value):
+def validate_partition_slice(partition_names, name, value):
     is_start = name == 'from'
     if value is None:
-        return 0 if is_start else len(partitions)
-    partition_names = [partition.name for partition in partitions]
+        return 0 if is_start else len(partition_names)
     if value not in partition_names:
         raise click.UsageError('invalid value {} for {}'.format(value, name))
     index = partition_names.index(value)
@@ -673,59 +812,79 @@ def pipeline_backfill_command(**kwargs):
 
 def execute_backfill_command(cli_args, print_fn, instance=None):
     instance = instance or DagsterInstance.get()
-    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
-    external_repository = get_external_repository_from_kwargs(cli_args)
+    repo_location = get_repository_location_from_kwargs(cli_args, instance)
+    external_repo = get_external_repository_from_repo_location(
+        repo_location, cli_args.get('repository')
+    )
 
-    # We should move this to use external repository
-    # https://github.com/dagster-io/dagster/issues/2556
-    recon_repo = recon_repo_from_external_repo(external_repository)
-    repo_def = recon_repo.get_definition()
+    external_pipeline = get_external_pipeline_from_external_repo(
+        external_repo, cli_args.get('pipeline'),
+    )
 
     noprompt = cli_args.get('noprompt')
 
-    pipeline_def = repo_def.get_pipeline(external_pipeline.name)
+    pipeline_partition_set_names = {
+        external_partition_set.name: external_partition_set
+        for external_partition_set in external_repo.get_external_partition_sets()
+        if external_partition_set.pipeline_name == external_pipeline.name
+    }
 
-    # Resolve partition set
-    all_partition_sets = repo_def.partition_set_defs + [
-        schedule_def.get_partition_set()
-        for schedule_def in repo_def.schedule_defs
-        if isinstance(schedule_def, PartitionScheduleDefinition)
-    ]
-
-    pipeline_partition_sets = [
-        x for x in all_partition_sets if x.pipeline_name == pipeline_def.name
-    ]
-    if not pipeline_partition_sets:
+    if not pipeline_partition_set_names:
         raise click.UsageError(
-            'No partition sets found for pipeline `{}`'.format(pipeline_def.name)
+            'No partition sets found for pipeline `{}`'.format(external_pipeline.name)
         )
     partition_set_name = cli_args.get('partition_set')
     if not partition_set_name:
-        if len(pipeline_partition_sets) == 1:
-            partition_set_name = pipeline_partition_sets[0].name
+        if len(pipeline_partition_set_names) == 1:
+            partition_set_name = next(iter(pipeline_partition_set_names.keys()))
         elif noprompt:
             raise click.UsageError('No partition set specified (see option `--partition-set`)')
         else:
             partition_set_name = click.prompt(
                 'Select a partition set to use for backfill: {}'.format(
-                    ', '.join(x.name for x in pipeline_partition_sets)
+                    ', '.join(x for x in pipeline_partition_set_names.keys())
                 )
             )
-    partition_set = next((x for x in pipeline_partition_sets if x.name == partition_set_name), None)
+
+    partition_set = pipeline_partition_set_names.get(partition_set_name)
+
     if not partition_set:
         raise click.UsageError('No partition set found named `{}`'.format(partition_set_name))
 
+    mode = partition_set.mode
+    solid_selection = partition_set.solid_selection
+
+    repo_handle = RepositoryHandle(
+        repository_name=external_repo.name,
+        repository_location_handle=repo_location.location_handle,
+    )
+
     # Resolve partitions to backfill
-    partitions = gen_partitions_from_args(partition_set, cli_args)
+    partition_names_or_error = repo_location.get_external_partition_names(
+        repo_handle, partition_set_name,
+    )
+
+    if isinstance(partition_names_or_error, ExternalPartitionExecutionErrorData):
+        raise DagsterBackfillFailedError(
+            'Failure fetching partition names for {partition_set_name}: {error_message}'.format(
+                partition_set_name=partition_set_name,
+                error_message=partition_names_or_error.error.message,
+            ),
+            serialized_error_info=partition_names_or_error.error,
+        )
+
+    partition_names = gen_partition_names_from_args(
+        partition_names_or_error.partition_names, cli_args
+    )
 
     # Print backfill info
-    print_fn('\n     Pipeline: {}'.format(pipeline_def.name))
-    print_fn('Partition set: {}'.format(partition_set.name))
-    print_fn('   Partitions: {}\n'.format(print_partition_format(partitions, indent_level=15)))
+    print_fn('\n     Pipeline: {}'.format(external_pipeline.name))
+    print_fn('Partition set: {}'.format(partition_set_name))
+    print_fn('   Partitions: {}\n'.format(print_partition_format(partition_names, indent_level=15)))
 
     # Confirm and launch
     if noprompt or click.confirm(
-        'Do you want to proceed with the backfill ({} partitions)?'.format(len(partitions))
+        'Do you want to proceed with the backfill ({} partitions)?'.format(len(partition_names))
     ):
 
         print_fn('Launching runs... ')
@@ -735,15 +894,42 @@ def execute_backfill_command(cli_args, print_fn, instance=None):
             PipelineRun.tags_for_backfill_id(backfill_id), get_tags_from_args(cli_args),
         )
 
-        for partition in partitions:
-            run = instance.create_run_for_pipeline(
-                pipeline_def=pipeline_def,
-                mode=partition_set.mode,
-                solids_to_execute=frozenset(partition_set.solid_selection)
-                if partition_set and partition_set.solid_selection
-                else None,
-                run_config=partition_set.run_config_for_partition(partition),
-                tags=merge_dicts(partition_set.tags_for_partition(partition), run_tags),
+        for partition_name in partition_names:
+            run_config_or_error = repo_location.get_external_partition_config(
+                repo_handle, partition_set_name, partition_name
+            )
+            if isinstance(run_config_or_error, ExternalPartitionExecutionErrorData):
+                raise DagsterBackfillFailedError(
+                    'Failure fetching run config for partition {partition_name} in {partition_set_name}: {error_message}'.format(
+                        partition_name=partition_name,
+                        partition_set_name=partition_set_name,
+                        error_message=run_config_or_error.error.message,
+                    ),
+                    serialized_error_info=run_config_or_error.error,
+                )
+
+            tags_or_error = repo_location.get_external_partition_tags(
+                repo_handle, partition_set_name, partition_name
+            )
+            if isinstance(tags_or_error, ExternalPartitionExecutionErrorData):
+                raise DagsterBackfillFailedError(
+                    'Failure fetching tags for partition {partition_name} in {partition_set_name}: {error_message}'.format(
+                        partition_name=partition_name,
+                        partition_set_name=partition_set_name,
+                        error_message=tags_or_error.error.message,
+                    ),
+                    serialized_error_info=tags_or_error.error,
+                )
+            run = _create_external_pipeline_run(
+                instance=instance,
+                repo_location=repo_location,
+                external_repo=external_repo,
+                external_pipeline=external_pipeline,
+                run_config=run_config_or_error.run_config,
+                mode=mode,
+                preset=None,
+                tags=merge_dicts(tags_or_error.tags, run_tags),
+                solid_selection=frozenset(solid_selection) if solid_selection else None,
             )
 
             instance.launch_run(run.run_id, external_pipeline)
@@ -752,7 +938,7 @@ def execute_backfill_command(cli_args, print_fn, instance=None):
 
         print_fn('Launched backfill job `{}`'.format(backfill_id))
     else:
-        print_fn(' Aborted!')
+        print_fn('Aborted!')
 
 
 pipeline_cli = create_pipeline_cli_group()
